/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

/*
Complex class declare for the GMP.
*/

#include <complex>

using namespace std;

class mpc_class {
public:
  mpf_class re, im;
  mpc_class() { re=0.0; im=0.0; }
  mpc_class(mpf_class i, mpf_class j) { re=i; im=j;}

  bool operator==(mpc_class &other);
  bool operator!=(mpc_class &other);

  void operator=(complex<double> ob1);
  void operator=(double ob1);
  void operator=(mpf_class ob1);

  friend mpc_class operator+(mpc_class ob1, mpc_class ob2);
  friend mpc_class operator+(mpc_class ob1, mpf_class ob2);
  friend mpc_class operator+(mpf_class ob1, mpc_class ob2);
  friend mpc_class operator+(mpc_class ob1, complex<double> ob2);

  friend mpc_class operator-(mpc_class ob1, mpc_class ob2);
  friend mpc_class operator-(mpc_class ob1, mpf_class ob2);
  friend mpc_class operator-(mpf_class ob1, mpc_class ob2);
  friend mpc_class operator-(mpf_class ob1, complex<double> ob2);

  friend mpc_class operator*(mpc_class ob1, mpc_class ob2);
  friend mpc_class operator*(mpc_class ob1, mpf_class ob2);
  friend mpc_class operator*(mpf_class ob1, mpc_class ob2);

  friend mpc_class operator/(mpc_class ob1, mpc_class ob2);
  friend mpc_class operator/(mpc_class ob1, mpf_class ob2);
  friend mpc_class operator/(mpf_class ob1, mpc_class ob2);

};

inline bool mpc_class::operator==(mpc_class &other)
{
  if ( (this->re == other.re) && (this->im == other.im) ) {
        return true;
  }
  else return false;
}

inline bool mpc_class::operator!=(mpc_class &other)
{
  if ( (this->re != other.re) || (this->im != other.im) ) {
        return true;
  }
  else return false;
}

inline void mpc_class::operator=(mpf_class ob1)
{
  re = ob1;
  im = 0.0;
}

inline void mpc_class::operator=(double ob1)
{
  re = ob1;
  im = 0.0;
}

inline void mpc_class::operator=(complex<double> ob1)
{
  re = ob1.real();
  im = ob1.imag();
}

inline mpc_class operator+(mpc_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re + ob2.re;
  tmp.im = ob1.im + ob2.im;
  return tmp;
} 

inline mpc_class operator+(mpf_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1 + ob2.re;
  tmp.im = ob2.im;
  return tmp;
} 

inline mpc_class operator+(mpc_class ob1, mpf_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re + ob2;
  tmp.im = ob1.im;
  return tmp;
} 

inline mpc_class operator+(mpc_class ob1, complex<double> ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re + ob2.real();
  tmp.im = ob1.im + ob2.imag();
  return tmp;
} 

inline mpc_class operator-(mpc_class ob1, complex<double> ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re - ob2.real();
  tmp.im = ob1.im - ob2.imag();
  return tmp;
} 

inline mpc_class operator-(mpc_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re - ob2.re;
  tmp.im = ob1.im - ob2.im;
  return tmp;
} 

inline mpc_class operator-(mpf_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1 - ob2.re;
  tmp.im = - ob2.im;
  return tmp;
} 

inline mpc_class operator-(mpc_class ob1, mpf_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re - ob2;
  tmp.im = ob1.im;
  return tmp;
} 

inline mpc_class operator*(mpc_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re * ob2.re - ob1.im * ob2.im;
  tmp.im = ob1.re * ob2.im + ob1.im * ob2.re;
  return tmp;
} 

inline mpc_class operator*(mpf_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1 * ob2.re;
  tmp.im = ob1 * ob2.im;
  return tmp;
} 

inline mpc_class operator*(mpc_class ob1, mpf_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re * ob2;
  tmp.im = ob1.im * ob2;
  return tmp;
} 

inline mpc_class operator/(mpc_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re = (ob1.re * ob2.re + ob1.im * ob2.im) / (ob2.re * ob2.re+ ob2.im * ob2.im);
  tmp.im = (ob1.im * ob2.re - ob1.re * ob2.im) / (ob2.re * ob2.re+ ob2.im * ob2.im);
  return tmp;
} 

inline mpc_class operator/(mpf_class ob1, mpc_class ob2)
{
  mpc_class tmp;
  tmp.re =   (ob1 * ob2.re) / (ob2.re * ob2.re+ ob2.im * ob2.im);
  tmp.im = - (ob1 * ob2.im) / (ob2.re * ob2.re+ ob2.im * ob2.im);
  return tmp;
} 

inline mpc_class operator/(mpc_class ob1, mpf_class ob2)
{
  mpc_class tmp;
  tmp.re = ob1.re / ob2;
  tmp.im = ob1.im / ob2;
  return tmp;
}

inline mpf_class abs(mpc_class ctmp)
{
  return sqrt(ctmp.re*ctmp.re + ctmp.im*ctmp.im);
}

inline mpc_class conjg(mpc_class ctmp)
{
  mpc_class ctmp2;
  ctmp2.re=ctmp.re;
  ctmp2.im=-ctmp.im;
  return ctmp2;
}

inline mpf_class real(mpc_class ctmp)
{
  return ctmp.re;
}

inline mpf_class imag(mpc_class ctmp)
{
  return ctmp.im;
}

